--=================================================================================================
-- file: reset.vhd
-- 
-- Internal DCok & RESET delays.
--
-- Author: Jay R. Lory
-- Revision: 1.0
--     Date: xx-DEC-96
--
-- Revision History:
-- Rev 1.0  xx-DEC-96   JRL   Original VHDL
-- 
--=================================================================================================
-- Define Libraries used
--
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE work.rtlpkg.ALL;
USE work.std_arith.ALL;

--=================================================================================================
-- Define I/O pORts
--
ENTITY Reset_FPGA IS PORT(
	POR_L          : IN    STD_LOGIC;  -- Power-On Reset
	ICT_L          : IN    STD_LOGIC;  -- In-Circuit Test tristate
	Clock          : IN    STD_LOGIC;  -- Clock input
  Mod_DCok       : IN    STD_LOGIC;  -- Module power status
  Bad_3V         : IN    STD_LOGIC;  -- +3.3V status
  Fan_Conn_L     : IN    STD_LOGIC;  -- Fan O.K.
  Good_2V        : IN    STD_LOGIC;  -- +2V status
  Mod_Reset_L    : IN    STD_LOGIC;  -- Module reset
	PLLbypass      : IN    STD_LOGIC;  -- PLL bypass
	RealSROMdata   : IN    STD_LOGIC;  -- Data from real SROM
	TestSROMdata   : IN    STD_LOGIC;  -- Data from connector
	SROMselect     : IN    STD_LOGIC;  -- SROM data select, 0 => real, 1 => test
  IRQ            : IN    STD_LOGIC_VECTOR(5 DOWNTO 0);   -- IRQ's
  TSUNspeed      : IN    STD_LOGIC_VECTOR(2 DOWNTO 0);   -- Tsunami speed
  CPUspeed       : IN    STD_LOGIC_VECTOR(2 DOWNTO 0);   -- CPU speed
	FBselect       : INOUT STD_LOGIC_VECTOR(1 DOWNTO 0);   -- Feedback select
	CPUirq         : INOUT STD_LOGIC_VECTOR(5 DOWNTO 0);   -- CPU IRQ inputs
	Fselect        : INOUT STD_LOGIC;  -- Frequency select
	SROMdata       : INOUT STD_LOGIC;  -- SROM data output
  Real_Reset_L   : INOUT STD_LOGIC;  -- System reset
  Vddq_Enable    : INOUT STD_LOGIC;  -- ?
  EV633_Enable_L : INOUT STD_LOGIC;  -- Turn on 2V DC-DC converter
  Real_DCok      : INOUT STD_LOGIC;  -- All power O.K.
  Reset_Delay    : INOUT STD_LOGIC;  -- Connection to external RC network for RESET delay (not used)
	OSCout         : INOUT STD_LOGIC); -- Internal Oscillator output (tie this pin to Clock pin)

  
-- Force part to CY7C3382A-XJC: 3.3V, 1K Gate, 96 logic cells, 68-pin PLCC.
ATTRIBUTE ORder_code OF Reset_FPGA:ENTITY IS "CY7C3382A-XJC";

-- Lock Pinout
ATTRIBUTE pin_numbers OF Reset_FPGA:ENTITY IS
--             GND:1
           "Bad_3V:2 " & 
       "Fan_Conn_L:3 " & 
          "Good_2V:4 " & 
--           ":5 " & 
          "Fselect:6 " & 
      "FBselect(0):7 " & 
      "FBselect(1):8 " & 
        "CPUirq(5):9 " & 
        "CPUirq(4):10 " & 
        "CPUirq(3):11 " & 
        "CPUirq(2):12 " & 
        "CPUirq(1):13 " & 
        "CPUirq(0):14 " & 
           "OSCout:15 " &
            "POR_L:16 " &
            "Clock:17 " &
--             VCC:18
            "ICT_L:19 " & 
        "PLLbypass:20 " & 
      "CPUspeed(2):21 " & 
      "CPUspeed(1):22 " & 
      "CPUspeed(0):23 " & 
     "TSUNspeed(2):24 " & 
     "TSUNspeed(1):25 " & 
     "TSUNspeed(0):26 " & 
           "IRQ(5):27 " & 
           "IRQ(4):28 " & 
           "IRQ(3):29 " & 
           "IRQ(2):30 " & 
           "IRQ(1):31 " & 
           "IRQ(0):32 " & 
         "Mod_DCok:33 " & 
      "Mod_Reset_L:34 " & 
--             GND:35
       "SROMselect:36 " & 
     "RealSROMdata:37 " & 
     "TestSROMdata:38 " & 
         "SROMdata:39 " & 
--             ":40 " & 
--             ":41 " & 
--             ":42 " & 
--             ":43 " & 
      "Reset_Delay:44 " & 
--             ":45 " & 
        "Real_DCok:46 " & 
   "EV633_Enable_L:47 " & 
      "Vddq_Enable:48 " & 
     "Real_Reset_L:49"; 
--             ":50 " & Dedicated Input, must be tied to either GND or VCC
--             ":51 " & Dedicated Input, must be tied to either GND or VCC 
--             VCC:52
--             ":53 " & Dedicated Input, must be tied to either GND or VCC 
--             ":54 " & Dedicated Input, must be tied to either GND or VCC 
--             ":55 " & 
--             ":56 " & 
--             ":57 " & 
--             ":58 " & 
--             ":59 " & 
--             ":60" 
--             ":61 " & 
--             ":62 " & 
--             ":63 " & 
--             ":64 " & 
--             ":65 " & 
--             ":66 " & 
--             ":67 " & 
--             ":68";
END Reset_FPGA;

--=================================================================================================
--
-- Define Reset Chip behaviOR
--
ARCHITECTURE reset_arch OF Reset_FPGA IS

--=========================
-- Define internal "wires"
--=========================
SIGNAL goe         : STD_LOGIC;
SIGNAL por         : STD_LOGIC;
SIGNAL evenff      : STD_LOGIC;
SIGNAL dcokff      : STD_LOGIC;
SIGNAL vddqen      : STD_LOGIC;
SIGNAL rst         : STD_LOGIC;
SIGNAL clk         : STD_LOGIC;
SIGNAL load        : STD_LOGIC;
SIGNAL count       : STD_LOGIC;
SIGNAL done        : STD_LOGIC;
SIGNAL cur_count   : STD_LOGIC_VECTOR(17 DOWNTO 0);
SIGNAL delaymux    : STD_LOGIC_VECTOR(cur_count'RANGE);
SIGNAL ydiv        : STD_LOGIC_VECTOR(2 DOWNTO 0);
SIGNAL frame       : STD_LOGIC_VECTOR(2 DOWNTO 0);
SIGNAL irqmux      : STD_LOGIC_VECTOR(5 DOWNTO 0);
SIGNAL fbsel       : STD_LOGIC_VECTOR(1 DOWNTO 0);
SIGNAL fsel        : STD_LOGIC;
SIGNAL osc         : STD_LOGIC;
SIGNAL delay1      : STD_LOGIC;
SIGNAL delay2      : STD_LOGIC;
SIGNAL delay3      : STD_LOGIC;
SIGNAL delay4      : STD_LOGIC;
SIGNAL delay5      : STD_LOGIC;
SIGNAL delay6      : STD_LOGIC;
SIGNAL delay7      : STD_LOGIC;
SIGNAL sdata       : STD_LOGIC;
SIGNAL delay       : STD_LOGIC;
SIGNAL reset_delay : STD_LOGIC;


SIGNAL one  : STD_LOGIC := '1';
SIGNAL zero : STD_LOGIC := '0';

-- Force VHDL compiler to keep the following gates
ATTRIBUTE dont_touch OF osc1:LABEL IS true;
ATTRIBUTE dont_touch OF osc2:LABEL IS true;
ATTRIBUTE dont_touch OF osc3:LABEL IS true;
ATTRIBUTE dont_touch OF osc4:LABEL IS true;
ATTRIBUTE dont_touch OF osc5:LABEL IS true;
ATTRIBUTE dont_touch OF osc6:LABEL IS true;
ATTRIBUTE dont_touch OF osc7:LABEL IS true;
ATTRIBUTE dont_touch OF osc8:LABEL IS true;
ATTRIBUTE dont_touch OF osc9:LABEL IS true;
ATTRIBUTE dont_touch OF b1:LABEL IS true;
ATTRIBUTE dont_touch OF b2:LABEL IS true;


-- Delay times based on 1.3uS internal OSC cycle. Delay times will vary with temperature!!
-- 
CONSTANT  DELAY30uS : INTEGER := 23;     -- 30uS DCok Delay
CONSTANT DELAY100mS : INTEGER := 76923;  -- 100mS RESET Delay

-- Define Reset State Machine as one-hot encoded
TYPE reset_states IS (cold,good3v,dcok,go,reset,pwrdwn);
ATTRIBUTE state_encoding OF reset_states:TYPE IS one_hot_one;
SIGNAL cur_state, nxt_state : reset_states;



--=================================================================================================
BEGIN

-- Buffer ICT pin
  b1:pafrag_a PORT MAP(ICT_L,zero,one,zero,one,zero,goe);

-- Force Vddq Enable active
	vddqen <= '1';

-- Force Power-On Reset on a High-Drive input with inverted output
	b2:paincell port map(POR_L,por,open);

--=================================================================================================
--
-- Reset STATE MACHINE
--
  reset_sm:PROCESS(por,cur_state,Mod_DCok,Bad_3V,Fan_Conn_L,Good_2V,done,Mod_Reset_L) BEGIN
     CASE cur_state IS
        WHEN cold =>
           IF (Mod_DCok = '1' AND Bad_3V = '0' AND Fan_Conn_L = '0') THEN
           	nxt_state <= good3v;
              load <= '0';
              count <= '0';
              delay <= '0';
           END IF;
        WHEN good3v =>
          	IF (Mod_DCok = '1' AND Bad_3V = '0' AND Fan_Conn_L = '0' AND Good_2V = '1') THEN
           	nxt_state <= dcok;
              load <= '1';
              delay <= '0';
           ELSIF (Mod_DCok = '0' OR Bad_3V = '1' OR Fan_Conn_L = '1') THEN
              nxt_state <= pwrdwn;
              load <= '1';
              delay <= '1';
           END IF;
        WHEN dcok =>
           load <= '0';
           count <= '1';
         	IF (Mod_DCok = '1' AND Bad_3V = '0' AND Fan_Conn_L = '0' AND Good_2V = '1' AND Mod_Reset_L = '1' AND done = '1') THEN
	        	   nxt_state <= go;
              count <= '0';
           ELSIF (Mod_DCok = '1' AND Bad_3V = '0' AND Fan_Conn_L = '0' AND Good_2V = '1' AND Mod_Reset_L = '0') THEN
	            nxt_state <= reset;
              count <= '0';
           ELSIF (Mod_DCok = '0' OR Bad_3V = '1' OR Fan_Conn_L = '1' OR Good_2V = '0') THEN
              nxt_state <= pwrdwn;
              load <= '1';
              delay <= '1';
           END IF;
        WHEN go =>
           IF (Mod_DCok = '1' AND Bad_3V = '0' AND Fan_Conn_L = '0' AND Good_2V = '1' AND Mod_Reset_L = '0') THEN
	        	   nxt_state <= reset;
           ELSIF (Mod_DCok = '0' OR Bad_3V = '1' OR Fan_Conn_L = '1' OR Good_2V = '0') THEN
             	nxt_state <= pwrdwn;
              load <= '1';
              delay <= '1';
           END IF;
        WHEN reset =>
           IF (Mod_DCok = '1' AND Bad_3V = '0' AND Fan_Conn_L = '0' AND Good_2V = '1' AND Mod_Reset_L = '1') THEN
	        	   nxt_state <= go;
           ELSIF (Mod_DCok = '0' OR Bad_3V = '1' OR Fan_Conn_L = '1' OR Good_2V = '0') THEN
           	nxt_state <= pwrdwn;
              load <= '1';
              delay <= '1';
           END IF;
        WHEN pwrdwn =>
           load <= '0';
           count <= '1';
        	IF (done = '1') THEN
        	   nxt_state <= cold;
              count <= '0';
           END IF;
     END CASE;
     IF (por = '1') THEN
        nxt_state <= cold;
     END IF;  
  END PROCESS;

-- Current State register
  curstate:PROCESS(Clock,por) BEGIN
     IF (por = '1') THEN
        cur_state <= cold;
     ELSIF RISING_EDGE(Clock) THEN
        cur_state <= nxt_state;
     END IF;
  END PROCESS;

-- Direct Reset State Machine Outputs
	rst <= '1' WHEN cur_state = go ELSE '0';

--=================================================================================================
--
-- EV633_Enable FF
--
  ev633jkff:PROCESS(Clock,por,nxt_state) BEGIN
     IF (por = '1') THEN
        evenff <= '1';
     ELSIF RISING_EDGE(Clock) THEN
        IF (nxt_state = good3v) THEN
           evenff <= '0';
        ELSIF (nxt_state = pwrdwn) THEN
           evenff <= '1';
        END IF;
     END IF;
  END PROCESS;
 
 
--=================================================================================================
--
-- Real_DCok FF
--
  dcokjkff:PROCESS(Clock,por,nxt_state) BEGIN
     IF (por = '1') THEN
        dcokff <= '0';
     ELSIF RISING_EDGE(Clock) THEN
        IF (nxt_state = dcok) THEN
           dcokff <= '1';
        ELSIF (nxt_state = cold) THEN
           dcokff <= '0';
        END IF;
     END IF;
  END PROCESS;
  
  
--=================================================================================================
--
-- Delay Down Counter & Mux
--
  dmux:PROCESS(delay) BEGIN
     IF (delay = '0') THEN
        delaymux <= to_std_logic_vector(DELAY100mS,cur_count'LEFT+1);
     ELSE
        delaymux <= to_std_logic_vector(DELAY30uS,cur_count'LEFT+1);
     END IF;
  END PROCESS;

  dncntr:PROCESS(Clock,por) BEGIN
     IF (por = '1') THEN
        cur_count <= (OTHERS => '0');
     ELSIF RISING_EDGE(Clock) THEN
        IF (load = '1') THEN
           cur_count <= delaymux;
        ELSIF (count = '1') THEN
           cur_count <= cur_count - 1;
        END IF;
     END IF;
  END PROCESS;

  done <= cur_count(cur_count'LEFT);

--=================================================================================================
--
-- ICT tristate
--
	ict:PROCESS(clk,evenff,dcokff,rst,vddqen,sdata,fbsel,fsel,irqmux) BEGIN
  	IF (goe = '1') THEN
  	   OSCout <= clk;
        EV633_Enable_L <= evenff;
        Real_DCok <= dcokff;
        Real_Reset_L <= rst;
        Vddq_Enable <= vddqen;
        SROMdata <= sdata;
	      FBselect <= fbsel;
	      Fselect <= fsel;
	      CPUirq <= irqmux;
     ELSE
        OSCout <= 'Z';
        EV633_Enable_L <= 'Z';
        Real_DCok <= 'Z';
        Real_Reset_L <= 'Z';
        Vddq_Enable <= 'Z';
        SROMdata <= 'Z';
	      FBselect <= (OTHERS => 'Z');
	      Fselect <= 'Z';
	      CPUirq <= (OTHERS => 'Z');
  	END IF;
	END PROCESS;


--=================================================================================================
--
-- SROM mux
--
  smux:PROCESS(SROMselect,RealSROMdata,TestSROMdata) BEGIN
     IF (SROMselect = '0') THEN
        sdata <= RealSROMdata;
     ELSE
        sdata <= TestSROMdata;
     END IF;
  END PROCESS;


--=================================================================================================
--
-- CPU IRQ mux
--
  imux:PROCESS(dcokff,ydiv,frame,IRQ) BEGIN
     IF (dcokff = '0') THEN
        irqmux(2 DOWNTO 0) <= ydiv;
        irqmux(5 DOWNTO 3) <= frame;
     ELSE
        irqmux <= IRQ;
     END IF;
  END PROCESS;

--=================================================================================================
--
-- External Reset RC Delay (disabled)
--
  reset_delay <= '1';
  rrcin:pabicell PORT MAP(reset_delay,one,zero,open,Reset_Delay);

--=================================================================================================
--
-- Internal Oscillator
--
	osc1:logico PORT MAP(osc,por,one,zero,one,zero,
                       one,zero,one,zero,zero,one,zero,one,
                       one,zero,one,zero,one,zero,delay1);

	osc2:logico PORT MAP(delay1,zero,one,zero,one,zero,
                       one,zero,one,zero,zero,one,zero,one,
                       one,zero,one,zero,one,zero,delay2);

	osc3:logico PORT MAP(delay2,zero,one,zero,one,zero,
                       one,zero,one,zero,zero,one,zero,one,
                       one,zero,one,zero,one,zero,osc);

	osc4:palcell PORT MAP(zero,delay3,zero,one,zero,one,zero,
                        one,zero,one,zero,zero,one,zero,one,
                        one,zero,one,zero,one,zero,
                        osc,por,open,open,delay3,open,open);

	osc5:palcell PORT MAP(zero,delay4,zero,one,zero,one,zero,
                        one,zero,one,zero,zero,one,zero,one,
                        one,zero,one,zero,one,zero,
                        delay3,por,open,open,delay4,open,open);

	osc6:palcell PORT MAP(zero,delay5,zero,one,zero,one,zero,
                        one,zero,one,zero,zero,one,zero,one,
                        one,zero,one,zero,one,zero,
                        delay4,por,open,open,delay5,open,open);

	osc7:palcell PORT MAP(zero,delay6,zero,one,zero,one,zero,
                        one,zero,one,zero,zero,one,zero,one,
                        one,zero,one,zero,one,zero,
                        delay5,por,open,open,delay6,open,open);

	osc8:palcell PORT MAP(zero,delay7,zero,one,zero,one,zero,
                        one,zero,one,zero,zero,one,zero,one,
                        one,zero,one,zero,one,zero,
                        delay6,por,open,open,delay7,open,open);

	osc9:palcell PORT MAP(zero,clk,zero,one,zero,one,zero,
                        one,zero,one,zero,zero,one,zero,one,
                        one,zero,one,zero,one,zero,
                        delay7,por,open,open,clk,open,open);


--=================================================================================================
--
-- Y-Divisor & Frame Lookup Table
--
	freq_table:PROCESS(dcokff,PLLbypass,TSUNspeed,CPUspeed) BEGIN
	  IF (PLLbypass = '0') THEN
        fbsel <= "00";
        fsel  <= '0';
        CASE TSUNspeed IS
--==============================================================
           WHEN o"0" =>
              CASE CPUspeed IS
                 WHEN o"0" | o"1" | o"2" =>    -- Tsunami = 50MHz, CPU = 400MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN o"3" | o"4" | o"5" =>    -- Tsunami = 50MHz, CPU = 500MHz
                    ydiv  <= o"5";
                    frame <= o"0";
                 WHEN OTHERS =>                -- Tsunami = 50MHz, CPU = 600MHz
                    ydiv  <= o"6";
                    frame <= o"0";
              END CASE;
--==============================================================
           WHEN o"1" =>
              CASE CPUspeed IS
                 WHEN o"0" | o"1" =>           -- Tsunami = 50MHz, CPU = 400MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN o"2" =>                  -- Tsunami = 58.3MHz, CPU = 466MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN o"3" | o"4" | o"5" =>    -- Tsunami = 50MHz, CPU = 500MHz
                    ydiv  <= o"5";
                    frame <= o"0";
                 WHEN OTHERS =>                -- Tsunami = 50MHz, CPU = 600MHz
                    ydiv  <= o"5";
                    frame <= o"0";
              END CASE;
--==============================================================
           WHEN o"2" =>
              CASE CPUspeed IS
                 WHEN o"0" | o"1" =>           -- Tsunami = 66.6MHz, CPU = 400MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"2" =>                  -- Tsunami = 58.3MHz, CPU = 466MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN o"3" =>                  -- Tsunami = 50MHz, CPU = 500MHz
                    ydiv  <= o"5";
                    frame <= o"0";
                 WHEN o"4" | o"5" =>           -- Tsunami = 66.6MHz, CPU = 533MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN o"6" =>                  -- Tsunami = 50MHz, CPU = 600MHz
                    ydiv  <= o"6";
                    frame <= o"0";
                 WHEN OTHERS =>                -- Tsunami = 66.6MHz, CPU = 666MHz
                    ydiv  <= o"5";
                    frame <= o"0";
              END CASE;
--==============================================================
           WHEN o"3" =>
              CASE CPUspeed IS
                 WHEN o"0" =>                  -- Tsunami = 66.6MHz, CPU = 400MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"1" =>                  -- Tsunami = 75MHz, CPU = 450MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"2" =>                  -- Tsunami = 58.3MHz, CPU = 466MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN o"3" =>                  -- Tsunami = 50MHz, CPU = 500MHz
                    ydiv  <= o"5";
                    frame <= o"0";
                 WHEN o"4" | o"5" =>           -- Tsunami = 66.6MHz, CPU = 533MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN o"6" =>                  -- Tsunami = 75MHz, CPU = 600MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN OTHERS =>                -- Tsunami = 66.6MHz, CPU = 666MHz
                    ydiv  <= o"5";
                    frame <= o"0";
              END CASE;
--==============================================================
           WHEN o"4" =>
              CASE CPUspeed IS
                 WHEN o"0" =>                  -- Tsunami = 66.6MHz, CPU = 400MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"1" =>                  -- Tsunami = 75MHz, CPU = 450MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"2" =>                  -- Tsunami = 58.3MHz, CPU = 466MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN o"3" =>                  -- Tsunami = 83.3MHz, CPU = 500MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"4" | o"5" =>           -- Tsunami = 66.6MHz, CPU = 533MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN o"6" =>                  -- Tsunami = 75MHz, CPU = 600MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN OTHERS =>                -- Tsunami = 83.3MHz, CPU = 666MHz
                    ydiv  <= o"4";
                    frame <= o"0";
              END CASE;
--==============================================================
           WHEN o"5" =>
              CASE CPUspeed IS
                 WHEN o"0" =>                  -- Tsunami = 66.6MHz, CPU = 400MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"1" =>                  -- Tsunami = 75MHz, CPU = 450MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"2" =>                  -- Tsunami = 58.3MHz, CPU = 466MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"3" =>                  -- Tsunami = 83.3MHz, CPU = 500MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"4" =>                  -- Tsunami = 66.6MHz, CPU = 533MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN o"5" =>                  -- Tsunami = 91.6MHz, CPU = 550MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"6" =>                  -- Tsunami = 75MHz, CPU = 600MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN OTHERS =>                -- Tsunami = 83.3MHz, CPU = 666MHz
                    ydiv  <= o"4";
                    frame <= o"0";
              END CASE;
--==============================================================
           WHEN o"6" =>
              CASE CPUspeed IS
                 WHEN o"0" =>                  -- Tsunami = 66.6MHz, CPU = 400MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"1" =>                  -- Tsunami = 75MHz, CPU = 450MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"2" =>                  -- Tsunami = 58.3MHz, CPU = 466MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"3" =>                  -- Tsunami = 83.3MHz, CPU = 500MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"4" =>                  -- Tsunami = 66.6MHz, CPU = 533MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN o"5" =>                  -- Tsunami = 91.6MHz, CPU = 550MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN o"6" =>                  -- Tsunami = 100MHz, CPU = 600MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN OTHERS =>                -- Tsunami = 83.3MHz, CPU = 666MHz
                    ydiv  <= o"4";
                    frame <= o"0";
              END CASE;
--==============================================================
           WHEN OTHERS =>
              CASE CPUspeed IS
                 WHEN o"0" | o"1" =>           -- Tsunami = 66.6MHz, CPU = 400MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                 WHEN o"2" =>                  -- Tsunami = 58.3MHz, CPU = 466MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN o"3" =>                  -- Tsunami = 50MHz, CPU = 500MHz
                    ydiv  <= o"5";
                    frame <= o"0";
                 WHEN o"4" | o"5" =>           -- Tsunami = 66.6MHz, CPU = 533MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                 WHEN o"6" =>                  -- Tsunami = 50MHz, CPU = 600MHz
                    ydiv  <= o"6";
                    frame <= o"0";
                 WHEN OTHERS =>                -- Tsunami = 66.6MHz, CPU = 666MHz
                    ydiv  <= o"5";
                    frame <= o"0";
              END CASE;
--==============================================================
        END CASE;
    ELSE
        fsel  <= NOT dcokff;
        CASE TSUNspeed IS
--==============================================================
           WHEN o"0" =>                              -- Tsunami = 50MHz, CPU = 400MHz
              ydiv  <= o"4";
              frame <= o"0";
              fbsel <= "11";
--==============================================================
           WHEN o"1" =>
              CASE CPUspeed IS
                 WHEN o"0" | o"1" | o"2" | o"3" =>   -- Tsunami = 58.5MHz, CPU = 351MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                    fbsel <= "10";
                 WHEN OTHERS =>                      -- Tsunami = 58.5MHz, CPU = 468MHz
                    ydiv  <= o"4";
                    frame <= o"0";
                    fbsel <= "11";
              END CASE;
--==============================================================
           WHEN o"2" =>                              -- Tsunami = 66.6MHz, CPU = 400MHz
              ydiv  <= o"3";
              frame <= o"0";
              fbsel <= "10";
--==============================================================
           WHEN o"3" =>
              CASE CPUspeed IS
                 WHEN o"0" =>                        -- Tsunami = 66.6MHz, CPU = 400MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                    fbsel <= "10";
                 WHEN OTHERS =>                      -- Tsunami = 75MHz, CPU = 450MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                    fbsel <= "10";
              END CASE;
--==============================================================
           WHEN o"4" =>
              CASE CPUspeed IS
                 WHEN o"0" | o"1" | o"2" =>          -- Tsunami = 66.6MHz, CPU = 400MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                    fbsel <= "10";
                 WHEN OTHERS =>                      -- Tsunami = 83.3MHz, CPU = 500MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                    fbsel <= "10";
              END CASE;
--==============================================================
           WHEN o"5" =>
              CASE CPUspeed IS
                 WHEN o"0" | o"1" | o"2" =>          -- Tsunami = 66.6MHz, CPU = 400MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                    fbsel <= "10";
                 WHEN OTHERS =>                      -- Tsunami = 83.3MHz, CPU = 500MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                    fbsel <= "10";
              END CASE;
--==============================================================
           WHEN o"6" =>
              CASE CPUspeed IS
                 WHEN o"0" | o"1" | o"2" =>          -- Tsunami = 66.6MHz, CPU = 400MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                    fbsel <= "10";
                 WHEN OTHERS =>                      -- Tsunami = 83.3MHz, CPU = 500MHz
                    ydiv  <= o"3";
                    frame <= o"0";
                    fbsel <= "10";
              END CASE;
--==============================================================
           WHEN OTHERS =>                            -- Tsunami = 66.6MHz, CPU = 400MHz
              ydiv  <= o"3";
              frame <= o"0";
              fbsel <= "10";
        END CASE;
     END IF;
  END PROCESS;
END reset_arch;

